/*
 * Copyright(C) OASIS(R) 2005,2009. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.
 */
package test.scaclient.systemproperty;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.net.URI;
import java.util.Properties;

import org.junit.Before;
import org.junit.Test;
import org.oasisopen.sca.NoSuchDomainException;
import org.oasisopen.sca.NoSuchServiceException;
import org.oasisopen.sca.SCARuntimeException;
import org.oasisopen.sca.client.SCAClient;
import org.oasisopen.sca.client.SCAClientFactory;

import com.helloservice.HelloService;
import com.othervendor.scaimpl.OtherVendorSCAClientFactoryImpl;
import com.othervendor.scaimpl.OtherVendorSCAClientImpl;
import com.vendor.scaimpl.VendorBootstrap;
import com.vendor.scaimpl.VendorSCAClientImpl;

public class TestSettingImplUsingSystemProperty
{
    private String serviceURI = "SomeHelloServiceURI";
    private URI domainURI;

    @Before
    public void setup() throws Exception
    {
        VendorBootstrap.bootstrap();
        domainURI = new URI("SomeDomainURI");
    }

    
    /**
     * Test that vendor provided implementation can be looked up by default.
     */
    @Test
    public void testGetVendorImpl() throws NoSuchServiceException, NoSuchDomainException {
        
        // Get SCAClient - will resolve to Vendors implementation
        final SCAClient scaClient = SCAClientFactory.newInstance();
        assertEquals(scaClient.getClass().getName(), VendorSCAClientImpl.class.getName());
        
        // Test that service lookup works
        final HelloService helloService = scaClient.getService(HelloService.class, serviceURI, domainURI);
        final String reply = helloService.sayHello("Mark");
        assertEquals("Hello Mark", reply);
    }
    
    
    /**
     * Test that vendor provided implementation can be overridden by setting a System Property for the lookup.
     */
    @Test
    public void testGetOtherVendorImplUsingSystemProperty() throws NoSuchServiceException, NoSuchDomainException {
        
        // Set System Property to Other Vendor to show it can be overridden
        System.setProperty(SCAClientFactory.class.getName(), OtherVendorSCAClientFactoryImpl.class.getName());

        // Get SCAClient - will resolve to Other Vendors implementation
        final SCAClient scaClient = SCAClientFactory.newInstance();
        assertEquals(scaClient.getClass().getName(), OtherVendorSCAClientImpl.class.getName());
        
        // Test that service lookup works
        final HelloService helloService = scaClient.getService(HelloService.class, serviceURI, domainURI);
        final String reply = helloService.sayHello("Mark");
        assertEquals("Hello Mark", reply);
    }

    /**
     * Test that vendor provided implementation can be overridden by setting a Property for the lookup.
     */
    @Test
    public void testGetOtherVendorImplUsingProperty() throws NoSuchServiceException, NoSuchDomainException {
        
        // Set Property to Other Vendor to show it can be overridden
        final Properties props = createPropertiesWithSCAFactoryImplName(OtherVendorSCAClientFactoryImpl.class.getName());
        
        // Get SCAClient - will resolve to Other Vendors implementation
        final SCAClient scaClient = SCAClientFactory.newInstance(props);
        assertEquals(scaClient.getClass().getName(), OtherVendorSCAClientImpl.class.getName());
        
        // Test that service lookup works
        final HelloService helloService = scaClient.getService(HelloService.class, serviceURI, domainURI);
        final String reply = helloService.sayHello("Mark");
        assertEquals("Hello Mark", reply);
    }
    
    /**
     * Tests what happens when the Property is set to a class that does not exist
     */
    @Test
    public void testPropertyForNoSuchSCAClientFactoryImpl() throws NoSuchServiceException, NoSuchDomainException {
        final String missingClassName = "this.class.does.not.exist";
        final Properties props = createPropertiesWithSCAFactoryImplName(missingClassName);
        
        try {
            SCAClientFactory.newInstance(props);
            fail("Should have thrown an exception");
        } catch (SCARuntimeException ex) {
            assertTrue(ex.getMessage().indexOf(missingClassName) != -1);
        }
    }

    /**
     * Tests what happens when the Property is set to a class that is not a SCAClientFactoryImpl
     */
    @Test
    public void testPropertyForNotSCAClientFactoryImpl() throws NoSuchServiceException, NoSuchDomainException {
        final String missingClassName = "java.lang.Object";
        final Properties props = createPropertiesWithSCAFactoryImplName(missingClassName);
        
        try {
            SCAClientFactory.newInstance(props);
            fail("Should have thrown an exception");
        } catch (SCARuntimeException ex) {
            assertTrue(ex.getMessage().indexOf(missingClassName) != -1);
        }
    }
    
    protected Properties createPropertiesWithSCAFactoryImplName(
            final String missingClassName)
    {
        final Properties props = new Properties();
        props.setProperty(SCAClientFactory.class.getName(), missingClassName);
        return props;
    }
    
}
